## ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####
# <pep8 compliant>

bl_info = {
    "name": "dxfWriter Settings",
    "author": "Thomas Lachmann [blender@automatic-brain.de]",
    "version": (0, 0, 8),
    "blender": (2, 73, 0),
    "location": "Render Layers(Properties window)",
    "description": "Settings for the freestyle dxfWriter.",
    "wiki_url": "",
    "tracker_url": "",
    "category": "Render",
}


import bpy
from bpy.types import Menu, Panel, UIList
from bpy.props import *

# properties

class class_dxfWriterSettings(bpy.types.PropertyGroup):
    select_by_visibility = BoolProperty(name="select_by_visibility", default=True)
    select_by_edge_types = BoolProperty(name="select_by_edge_types", default=True)
    select_by_face_marks = BoolProperty(name="select_by_face_marks", default=False)
    select_by_group = BoolProperty(name="select_by_group", default=False)
    select_by_image_border =  BoolProperty(name="select_by_image_border", default=True)
    
    visibility = EnumProperty(name='visibility', items=[('VISIBLE','Visible','',1),('HIDDEN','Hidden','',2),('RANGE','Qi Range','',3)])
    
    qi_start = IntProperty(name="Start", default=0)
    qi_end = IntProperty(name="End", default=100)
    
    edge_type_negation	= EnumProperty(items=[('INCLUSIVE','Inclusive','',1),('EXCLUSIVE','Exclusive','',2)])
    edge_type_combination = EnumProperty (items=[('OR','Logical OR','',1),('AND','Logical AND','',2)])
    
    select_silhouette = BoolProperty(name='Silhouette',default=True)
    select_border = BoolProperty(name='Border',default=True)
    select_contour = BoolProperty(name='Contour',default=False)
    select_suggestive_contour = BoolProperty(name='Suggestive Contour',default=False)
    select_ridge_valley = BoolProperty(name='Ridge & Valley', default=False)
    select_crease = BoolProperty(name='Crease',default=True)
    select_edge_mark = BoolProperty(name='Edge Mark',default=False)
    select_external_contour = BoolProperty(name='External Contour',default=False)
    select_material_boundary = BoolProperty(name='Material Boundary',default=False)

    exclude_silhouette = BoolProperty(default=False)
    exclude_border = BoolProperty(default=False)
    exclude_contour = BoolProperty(default=False)
    exclude_suggestive_contour = BoolProperty(default=False)
    exclude_ridge_valley = BoolProperty(default=False)
    exclude_crease = BoolProperty(default=False)
    exclude_edge_mark = BoolProperty(default=False)
    exclude_external_contour = BoolProperty(default=False)
    exclude_material_boundary = BoolProperty(default=False)
    
    
    face_mark_negation	= EnumProperty(items=[('INCLUSIVE','Inclusive','',1),('EXCLUSIVE','Exclusive','',2)])
    face_mark_condition	= EnumProperty(items=[('ONE','One Face','',1),('BOTH','Both Faces','',2)])
    
    #group = StringProperty()
    group = IntProperty(default=-1)
    group_negation = EnumProperty(items=[('INCLUSIVE','Inclusive','',1),('EXCLUSIVE','Exclusive','',2)])
   
    use_chaining = BoolProperty(name='use_chaining',default=False)
    use_same_object = BoolProperty(name='use_same_object',default=True)
    
    coloring = EnumProperty(items=[('MATERIAL','Material Color','Get color from objects material',1),('RENDERED','Rendered color','Get coloring from rendered image',2)])


# Render layer properties

class RenderLayerFreestyleDXFButtonsPanel():
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "render_layer"
    # COMPAT_ENGINES must be defined in each subclass, external engines can add themselves here

    @classmethod
    def poll(cls, context):
        scene = context.scene
        rd = context.scene.render
        with_freestyle = bpy.app.build_options.freestyle

        return (scene and with_freestyle and rd.use_freestyle
            and rd.layers.active and(scene.render.engine in cls.COMPAT_ENGINES))


class RenderLayerFreestyleDXFEditorButtonsPanel(RenderLayerFreestyleDXFButtonsPanel):
    # COMPAT_ENGINES must be defined in each subclass, external engines can add themselves here

    @classmethod
    def poll(cls, context):
        if not super().poll(context):
            return False
        rl = context.scene.render.layers.active
        dxfWriter_active = False
        if rl.freestyle_settings.mode == 'SCRIPT':
            for module in rl.freestyle_settings.modules:
                if module.script != None:
                    if 'dxfWriter.py' in module.script.name:
                        dxfWriter_active = True
                        break
        return dxfWriter_active


class RENDERLAYER_PT_freestyle_dxf_lineset(RenderLayerFreestyleDXFEditorButtonsPanel, Panel):
    bl_label = "Freestyle Line Set for dxfWriter"
    COMPAT_ENGINES = {'BLENDER_RENDER'}

    def draw_edge_type_buttons(self, box, lineset, edge_type):
        # property names
        select_edge_type = "select_" + edge_type
        exclude_edge_type = "exclude_" + edge_type
        # draw edge type buttons
        row = box.row(align=True)
        row.prop(lineset, select_edge_type)
        sub = row.column(align=True)
        sub.prop(lineset, exclude_edge_type, text="", icon='X')
        sub.active = getattr(lineset, select_edge_type)

    def draw(self, context):
        layout = self.layout

        rd = context.scene.render
        rl = rd.layers.active

        lineset = context.scene.dxfWriter_settings

        layout.active = rl.use_freestyle

        col = layout.column()
        col.label(text="Selection By:")
        row = col.row(align=True)
        row.prop(lineset, "select_by_visibility", text="Visibility", toggle=True)
        row.prop(lineset, "select_by_edge_types", text="Edge Types", toggle=True)
        row.prop(lineset, "select_by_face_marks", text="Face Marks", toggle=True)
        row.prop(lineset, "select_by_group", text="Group", toggle=True)
        row.prop(lineset, "select_by_image_border", text="Image Border", toggle=True)	

        if lineset.select_by_visibility:
                col.label(text="Visibility:")
                row = col.row(align=True)
                row.prop(lineset, "visibility", expand=True)
        if lineset.visibility == 'RANGE':
                row = col.row(align=True)
                row.prop(lineset, "qi_start")
                row.prop(lineset, "qi_end")

        if lineset.select_by_edge_types:
                col.label(text="Edge Types:")
                row = col.row()
                row.prop(lineset, "edge_type_negation", expand=True)
                row.prop(lineset, "edge_type_combination", expand=True)

                split = col.split()

                sub = split.column()
                self.draw_edge_type_buttons(sub, lineset, "silhouette")
                self.draw_edge_type_buttons(sub, lineset, "border")
                self.draw_edge_type_buttons(sub, lineset, "contour")
                self.draw_edge_type_buttons(sub, lineset, "suggestive_contour")
                self.draw_edge_type_buttons(sub, lineset, "ridge_valley")

                sub = split.column()
                self.draw_edge_type_buttons(sub, lineset, "crease")
                self.draw_edge_type_buttons(sub, lineset, "edge_mark")
                self.draw_edge_type_buttons(sub, lineset, "external_contour")
                self.draw_edge_type_buttons(sub, lineset, "material_boundary")

        if lineset.select_by_face_marks:
                col.label(text="Face Marks:")
                row = col.row()
                row.prop(lineset, "face_mark_negation", expand=True)
                row.prop(lineset, "face_mark_condition", expand=True)

        if lineset.select_by_group:
                col.label(text="Group:")
                row = col.row()
                #row.prop(lineset, "group", text="")
                row.operator_menu_enum("group.select_group", "dxfWriter_select_group", text=WriteSelectGroup.bl_label)
                row.prop(lineset, "group_negation", expand=True)

        col.label(text="Chaining:")
        row = col.row()
        row.prop(lineset, "use_chaining", text="Enable Chaining")
        sub = col.row()
        sub.active = lineset.use_chaining
        sub.prop(lineset, "use_same_object", text="Same Object")
            
        col.label(text="Coloring:")
        row = col.row()
        row.prop(lineset, "coloring", expand=True)


class WriteSelectGroup(bpy.types.Operator):
    bl_idname = "group.select_group"
    bl_label = ""


    def avail_groups(self,context):
        items = [(str(i),x.name,x.name) for i,x in enumerate(bpy.data.groups)]
        items.insert(0,('-1','None','None'))
        return items
    dxfWriter_select_group = bpy.props.EnumProperty(items = avail_groups, name = "Available Groups")
    
    def execute(self,context):
        if int(self.dxfWriter_select_group) == -1:
            self.__class__.bl_label = ''
        else:
            self.__class__.bl_label = bpy.data.groups[int(self.dxfWriter_select_group)].name
            context.scene.dxfWriter_settings.group = int(self.dxfWriter_select_group)
        return {'FINISHED'}
        

def register():
    bpy.utils.register_class(class_dxfWriterSettings)
    bpy.types.Scene.dxfWriter_settings = bpy.props.PointerProperty(type=class_dxfWriterSettings)
    bpy.utils.register_module(__name__)


def unregister():
    bpy.utils.unregister_module(__name__)
    del bpy.types.Scene.dxfWriter_settings
    
if __name__ == "__main__":  # only for live edit.
    register()
